import logging

import os
import cv2
import numpy as np
from tensorflow.keras.utils import to_categorical
from tensorflow.keras.models import Sequential
from tensorflow.keras.layers import Dense, Dropout, Flatten, Conv2D, MaxPooling2D
from sklearn.model_selection import train_test_split

IMG_SIZE = 28
TEST_SIZE_FACTOR = 0.1
# TODO maybe use an ENUM (here and armor_dataset_factory) because no 6 makes lost space in array of classes
N_CLASSES = 9  # there is no 6s or 9s


# from polystar.common.image_pipeline.image_pipeline import ImagePipeline
# from polystar.common.image_pipeline.models.random_model import RandomModel
# from research.armor_digit.armor_digit_pipeline_reporter_factory import ArmorDigitPipelineReporterFactory
# from research_common.dataset.twitch.twitch_roco_datasets import TwitchROCODataset
# from research_common.dataset.union_dataset import UnionDataset

# if __name__ == "__main__":
#     logging.getLogger().setLevel("INFO")
#
#     reporter = ArmorDigitPipelineReporterFactory.from_roco_datasets(
#         train_roco_datasets=[TwitchROCODataset.TWITCH_470151286, TwitchROCODataset.TWITCH_470150052],
#         test_roco_datasets=[TwitchROCODataset.TWITCH_470152289],
#     )
#
#     random_pipeline = ImagePipeline(model=RandomModel(), custom_name="random")
#
#     reporter.report([random_pipeline], evaluation_short_name="baseline")


def load_digits_img_to_data():
    # TODO add a model for digits_img in common/polystar/common/models
    # TODO create a from_dataset (see armor_dataset_factory) to read digits_img from digits_found folder
    # TODO maybe use an ENUM for labels (here and armor_dataset_factory) because no 6 makes lost space in classes' array
    # TODO but before, we use this hardcoded path...
    path = '.\\..\\..\\..\\dataset\\dji_roco\\robomaster_Final Tournament\\digits_found'
    digits_found = os.listdir(path)
    all_features = []
    all_labels = []

    for i, img_file in enumerate(digits_found):
        img_file_name = os.path.join(path, img_file)
        digit_img = cv2.imread(img_file_name)
        digit_res = cv2.resize(digit_img, dsize=(IMG_SIZE, IMG_SIZE), interpolation=cv2.INTER_CUBIC)
        digit_res = cv2.cvtColor(digit_res, cv2.COLOR_BGR2GRAY)
        all_features.append(digit_res)
        all_labels.append(img_file[-5])

    # split dataset into training and test
    features_train, features_test, labels_train, labels_test = train_test_split(
        all_features, all_labels, test_size=TEST_SIZE_FACTOR)

    return np.array(features_train), np.array(labels_train), np.array(features_test), np.array(labels_test)


def training_model(features_train, labels_train, features_test, labels_test):
    # Source: https://www.sitepoint.com/keras-digit-recognition-tutorial/
    # Formatting
    features_train = features_train.reshape(features_train.shape[0], IMG_SIZE, IMG_SIZE, 1)
    features_test = features_test.reshape(features_test.shape[0], IMG_SIZE, IMG_SIZE, 1)
    label_train = to_categorical(labels_train, N_CLASSES)
    label_test = to_categorical(labels_test, N_CLASSES)

    # Design Model
    model = Sequential()
    model.add(Conv2D(32, kernel_size=(3, 3),
                     activation='relu',
                     input_shape=(IMG_SIZE, IMG_SIZE, 1)))
    model.add(Conv2D(64, (3, 3), activation='relu'))
    model.add(MaxPooling2D(pool_size=(2, 2)))
    # randomly drop units to avoid overfitting
    model.add(Dropout(0.25))
    model.add(Flatten())
    model.add(Dense(128, activation='relu'))
    # randomly drop units to avoid overfitting
    model.add((Dropout(0.5)))
    # because we have a number of pre-decided classes
    model.add(Dense(N_CLASSES, activation='softmax'))

    # Compile and Train Model
    model.compile(loss='categorical_crossentropy',
                  optimizer='adam',
                  metrics=['accuracy'])

    batch_size = 128
    epochs = 20

    model.fit(features_train, label_train,
              batch_size=batch_size,
              epochs=epochs,
              verbose=1,
              validation_data=(features_test, label_test))
    score = model.evaluate(features_test, label_test, verbose=0)
    print('Test loss:', score[0])
    print('Test accuracy:', score[1])
    model.save("test_model.h5")  # TODO change resulting file name (and erase file test_model.h5)


if __name__ == "__main__":
    digits_features_train, digits_labels_train, digits_features_test, digits_labels_test = load_digits_img_to_data()
    for img in digits_labels_test:
        print('test', img)
    training_model(digits_features_train, digits_labels_train, digits_features_test, digits_labels_test)
