from pathlib import Path
from typing import List, Iterable, Tuple
import cv2

import matplotlib.pyplot as plt

from polystar.common.models.image import Image
from polystar.common.models.image_annotation import ImageAnnotation
from polystar.common.models.object import ObjectType, Armor
from polystar.common.target_pipeline.objects_validators.type_object_validator import TypeObjectValidator
from research_common.dataset.dji.dji_roco_datasets import DJIROCODataset
from research_common.dataset.roco_dataset import ROCODataset


class ArmorDatasetFactory:
    @staticmethod
    def from_image_annotation(image_annotation: ImageAnnotation) -> Iterable[Tuple[Image, Armor, int, Path]]:
        img = image_annotation.image
        armors: List[Armor] = TypeObjectValidator(ObjectType.Armor).filter(image_annotation.objects, img)
        for i, obj in enumerate(armors):
            croped_img = img[obj.y: obj.y + obj.h, obj.x: obj.x + obj.w]
            yield croped_img, obj, i, image_annotation.image_path

    @staticmethod
    def from_dataset(dataset: ROCODataset) -> Iterable[Tuple[Image, Armor, int, Path]]:
        for image_annotation in dataset.image_annotations:
            for rv in ArmorDatasetFactory.from_image_annotation(image_annotation):
                yield rv


def save_digits_img():
    for j, (digit_img, armor, k, p) in enumerate(ArmorDatasetFactory.from_dataset(DJIROCODataset.Final)):
        # we ignore the 6s, because the format of their pictures is not accurate and we don't need them

        # usage:
        #       - remove the two if (line 40 and 53-54) to load all images
        # or
        #       - load bunch of images (not all at the same time, depending if you just want to do some tests)
        #       1. start with j >= 0 at the top and j >= 500 at the bottom to load the 500 first images
        #       2. increse these numbers as you load other images (to save time and not load images twice)


        # watchout:
        # some images have chinese symbols in their name, you will either need to change their name of find another workaround

        if armor.numero != 6 and j >= 5925:  # TODO change j value depending where you are in saving (to save time)
            # if j >= 50:
            #     break
            # TODO avoid getting image with chinese symbols in their path
            try:
                print(armor.color, armor.numero, k, 'in', p)
                plt.imshow(digit_img)
                plt.savefig(str(p.parents[1]) + '\\digits_found\\digit' + str(j) + '_' + armor.color.name +
                            '_' + str(armor.numero) + '.png')
                plt.clf()
            except ValueError:
                # because some digit's pictures have zero-array values
                continue
        if j >= 5940:
            break


if __name__ == "__main__":
    save_digits_img()
